<?php
/**
 * @file
 * Definition of EntitySyncPacket class.
 */

class EntitySyncPacket {

  public $uniqueField;
  public $uniqueValue;
  public $entityType;
  public $bundle;
  public $fields;
  public $checksum;

  /**
   * When both parameters are given, a packet is created from the given entity
   * using the unique field as a key.
   *
   * @param $entity
   *   object|NULL DrupalEntityWrapper object
   * @param $unique
   *   object|NULL
   * @param $fields
   *   array Array of fields to sync
   */
  public function __construct($entity = NULL, $unique = NULL, $fields = array()) {

    if ($entity == NULL) {
      return;
    }

    $info        = $unique->info();
    $field_name  = $info['name'];
    $entity_type = $entity->type();
    $bundle      = $entity->getBundle();

    // Prepare syncing.
    $this->uniqueField = $field_name;
    $this->uniqueValue = $unique->value();
    $this->entityType  = $entity_type;
    $this->bundle      = $bundle;
    $this->fields      = array();

    // Collect filtered fields.
    $filter_fields = array();
    $entity_info = $entity->entityInfo();

    foreach ($entity_info['entity keys'] as $key) {
      $filter_fields[] = $key;
    }

    // Collect the whole entity.
    if (isset($fields['-all-'])) {
      $fields = $entity->getPropertyInfo();
      $fields = array_keys($fields);
    }

    $entity_raw = $entity->raw();

    foreach ($fields as $field_name) {
      if (!in_array($field_name, $filter_fields) &&
          isset($entity_raw->$field_name)) {
        $this->fields[$field_name] = $entity_raw->$field_name;
      }
    }
  }

  /**
   * Decode a json object to this packet.
   *
   * @return boolean
   *   TRUE or FALSE signifying if the packet is valid (trustworthy) or not
   */
  public function decode($json) {
    $obj = drupal_json_decode($json);
    foreach ($obj as $key => $value) {
      $this->$key = $value;
    }
    return $this->validate();
  }

  /**
   * Get encoded version of this packet.
   *
   * @return string
   *   Encoded packet
   */
  public function encoded() {
    if (isset($this->checksum)) {
      unset($this->checksum);
    }
    $secret = variable_get('entitysync_shared_secret', 'undefined');
    $this->checksum = md5($secret . serialize($this->fields));
    return drupal_json_encode($this);
  }

  /**
   * Fetch a Drupal entity for this packet.
   *
   * @return object|boolean
   *   Drupal entity object or FALSE in case of an error
   */
  public function fetchEntity() {

    $entity_field_query = new EntityFieldQuery();

    $entity_field_query->entityCondition('entity_type', $this->entityType)
      ->entityCondition('bundle', $this->bundle)
      ->propertyCondition($this->uniqueField, $this->uniqueValue);

    $entities = $entity_field_query->execute();
    $ids = array_keys($entities[$this->entityType]);
    $entities = entity_load($this->entityType, $ids);

    $number_of_entities = count($entities);

    if ($number_of_entities == 0) {
      // Create a new one.
      return (object) NULL;
    }
    elseif ($number_of_entities == 1) {
      // Update existing.
      return array_shift($entities);
    }
    // Generate error.
    return FALSE;
  }

  /**
   * Validate the packet.
   *
   * @return boolean
   *   TRUE if the packet is OK, FALSE if not
   */
  protected function validate() {
    $checksum = $this->checksum;
    unset($this->checksum);
    $secret = variable_get('entitysync_shared_secret', 'undefined');
    return $checksum == md5($secret . serialize($this->fields));
  }

  /**
   * Get the queue identifier for this packet
   *
   * @return string
   *   String which identifies the entity
   */
  public function getQueueIdentifier() {
    return $this->entityType . ':'
      . $this->bundle . ':'
      . $this->uniqueField . ':'
      . $this->uniqueValue;
  }

};
